@@ -2,18 +2,22 @@
 use strict;
 
 use Config;
+use Carp;
 use Cwd;
 use File::Spec;
 use File::Path;
+use File::Copy;
 use Data::Dumper;
-use FindBin;
-use lib File::Spec->catdir( $FindBin::Bin, 'lib' );
-use lib $FindBin::Bin;
+use File::Basename;
+my $findbin;
+BEGIN { $findbin = dirname $0 }
+use lib File::Spec->catdir( $findbin, 'lib' );
+use lib $findbin;
 use Test::Smoke::Util qw( do_pod2usage );
 
-# $Id: configsmoke.pl 482 2003-10-20 05:36:13Z abeltje $
+# $Id: configsmoke.pl 872 2005-07-21 16:53:21Z abeltje $
 use vars qw( $VERSION $conf );
-$VERSION = '0.034';
+$VERSION = '0.049';
 
 use Getopt::Long;
 my %options = ( 
@@ -23,11 +27,12 @@ my %options = (
     default => undef,
     prefix  => undef,
     oldcfg  => 0,
+    usedft  => undef,
 );
 my $myusage = "Usage: $0 -p <prefix>[ -d <defaultsprefix>]";
 GetOptions( \%options, 
     'config|c=s', 'jcl|j=s', 'log|l=s', 
-    'prefix|p=s', 'default|d=s',
+    'prefix|p=s', 'default|d:s', 'usedft|des',
 
     'help|h', 'man',
 ) or do_pod2usage( verbose => 1, myusage => $myusage );
@@ -57,13 +62,27 @@ if ( $@ || $options{default} ) {
         unless $@;
 } 
 
+# -des will only work fully when $options{oldcfg}
+unless ( $options{oldcfg} ) {
+    defined $options{usedft}
+        and print "Option '-des' not fully functional!\n";
+} else {
+    # -d like in ./Configure, works for oldcfg only!
+    $options{usedft} ||= defined $options{default} &&
+                         $options{default} eq "";
+}
+
 =head1 NAME
 
 configsmoke.pl - Create a configuration for B<smokeperl.pl>
 
 =head1 SYNOPSIS
 
-   $ perl configsmoke.pl -p <prefix>[ -d <defaultsprefix>]
+    $ perl configsmoke.pl -p <prefix>[ -d <defaultsprefix>]
+
+or regenerate from previous _config:
+
+    $ perl configsmoke.pl -p <prefix> -des 
 
 =head1 OPTIONS
 
@@ -75,9 +94,12 @@ Current options:
   -l logprefix      When ommited 'perlcurrent' is used
   -p prefix         Set -c and -j and -l at once
 
+  -des              confirm all answers (needs previous _config)
+
 =cut
 
 sub is_win32() { $^O eq 'MSWin32' }
+sub is_vms()   { $^O eq 'VMS'     }
 
 my %config = ( perl_version => $conf->{perl_version} || '5.9.x' );
 
@@ -86,6 +108,7 @@ my @mailers = sort keys %mailers;
 my @syncers = get_avail_sync();
 my $syncmsg = join "\n", @{ { 
     rsync    => "\trsync - Use the rsync(1) program [preferred]",
+    ftp      => "\tftp - Use Net::FTP to sync from APC [!slow!]",
     copy     => "\tcopy - Use File::Copy to copy from a local directory",
     hardlink => "\thardlink - Copy from a local directory using link()",
     snapshot => "\tsnapshot - Get a snapshot using Net::FTP (or LWP::Simple)",
@@ -93,15 +116,36 @@ my $syncmsg = join "\n", @{ {
 my @untars = get_avail_tar();
 my $untarmsg = join "", map "\n\t$_" => @untars;
 
+my %vdirs = map {
+    my $vdir = $_;
+    is_vms and $vdir =~ tr/.//d;
+    ( $_ => $vdir )
+} qw( 5.5.x 5.6.2 5.8.x );
+ 
 my %versions = (
+    '5.5.x' => { source => 'ftp.linux.activestate.com::perl-5.005xx',
+                 server => 'ftp.funet.fi',
+                 sdir   => '/pub/languages/perl/snap/5.5.x',
+                 sfile  => '',
+                 pdir   => '/pub/staff/gsar/APC/perl-5.005xx-diffs',
+                 cfg    => 'perl55x.cfg',
+                 ddir   => File::Spec->catdir( cwd(), File::Spec->updir,
+                                               "perl-$vdirs{'5.5.x'}" ),
+                 text   => 'Perl 5.005 MAINT',
+                 is56x  => 1},
     '5.6.2' => { source => 'ftp.linux.activestate.com::perl-5.6.2',
                  server => 'http://rgarciasuarez.free.fr',
                  sdir   => '/snap',
                  sfile  => 'perl562-21463.tar.gz',
                  pdir   => '/pub/staff/gsar/APC/perl-5.6.2-diffs',
-                 ddir   => File::Spec->rel2abs( 
-                               File::Spec->catdir( File::Spec->updir,
-                                                   'perl-5.6.2' ) ),
+                 ddir   => File::Spec->catdir( cwd(), File::Spec->updir,
+                                               "perl-$vdirs{'5.6.2'}" ),
+                 ftphost => 'ftp.linux.activestate.com',
+                 ftpusr  => 'anonymous',
+                 ftppwd  => 'smokers@perl.org',
+                 ftpsdir => '/pub/staff/gsar/APC/perl-5.6.2',
+                 ftpcdir => '/pub/staff/gsar/APC/perl-5.6.2-diffs',
+                 
                  cfg    => 'perl562.cfg',
                  text   => 'Perl 5.6.2-to-be',
                  is56x  => 1 },
@@ -110,24 +154,34 @@ my %versions = (
                  sdir   => '/pub/languages/perl/snap/5.8.x',
                  sfile  => '',
                  pdir   => '/pub/staff/gsar/APC/perl-5.8.x-diffs',
-                 ddir   => File::Spec->rel2abs( 
-                               File::Spec->catdir( File::Spec->updir,
-                                                   'perl-5.8.x' ) ),
-                 text   => 'Perl 5.8.2-to-be',
-                 cfg    => ( $^O eq 'MSWin32' 
-                        ? 'w32current.cfg' :'perlcurrent.cfg' ),
+                 ddir   => File::Spec->catdir( cwd(), File::Spec->updir,
+                                               "perl-$vdirs{'5.8.x'}" ),
+                 ftphost => 'ftp.linux.activestate.com',
+                 ftpusr  => 'anonymous',
+                 ftppwd  => 'smokers@perl.org',
+                 ftpsdir => '/pub/staff/gsar/APC/perl-5.8.x',
+                 ftpcdir => '/pub/staff/gsar/APC/perl-5.8.x-diffs',
+                 
+                 text   => 'Perl 5.8 MAINT',
+                 cfg    => ( is_win32 ? 'w32current.cfg'
+                           : is_vms ? 'vmsperl.cfg' : 'perl58x.cfg' ),
                  is56x  => 0 },
     '5.9.x' => { source => 'ftp.linux.activestate.com::perl-current',
                  server => 'ftp.funet.fi',
                  sdir   => '/pub/languages/perl/snap/5.9.x',
                  sfile  => '',
                  pdir   => '/pub/staff/gsar/APC/perl-current-diffs',
-                 ddir   => File::Spec->rel2abs( 
-                               File::Spec->catdir( File::Spec->updir,
-                                                   'perl-current' ) ),
-                 text   => 'Perl 5.10.0-to-be',
-                 cfg    => ( $^O eq 'MSWin32' 
-                        ? 'w32current.cfg' :'perlcurrent.cfg' ),
+                 ddir   => File::Spec->catdir( cwd(), File::Spec->updir,
+                                               'perl-current' ),
+                 ftphost => 'ftp.linux.activestate.com',
+                 ftpusr  => 'anonymous',
+                 ftppwd  => 'smokers@perl.org',
+                 ftpsdir => '/pub/staff/gsar/APC/perl-current',
+                 ftpcdir => '/pub/staff/gsar/APC/perl-current-diffs',
+                 
+                 text   => 'Perl 5.10 to-be',
+                 cfg    => ( is_win32 ? 'w32current.cfg'
+                           : is_vms ? 'vmsperl.cfg' : 'perlcurrent.cfg' ),
                  is56x  => 0 },
 );
 my @pversions = sort keys %versions;
@@ -154,8 +208,8 @@ my %opt = (
         msg => "Where would you like the new source-tree?
 \tThis directory is also used as the build directory.",
         alt => [ ],
-        dft => File::Spec->rel2abs( File::Spec->catdir( File::Spec->updir,
-                                                        'perl-current' ) ),
+        dft => File::Spec->catdir( File::Spec->rel2abs( File::Spec->updir ),
+                                   'perl-current' ),
         chk => '.+',
     },
     use_old => {
@@ -168,8 +222,9 @@ my %opt = (
     cfg => {
         msg => 'Which build-configuration file would you like to use?',
         alt => [ ],
-        dft => File::Spec->rel2abs( is_win32
-                                    ? 'w32current.cfg' : 'perlcurrent.cfg' ),
+        dft => File::Spec->rel2abs(
+                ( is_win32 ? 'w32current.cfg'
+                : is_vms ? 'vmsperl.cfg' : 'perlcurrent.cfg' ) ),
     },
     change_cfg => {
         msg => undef, # Set later...
@@ -209,8 +264,8 @@ my %opt = (
     forest_hdir => {
         msg => 'Where would you like the intermediate source-tree?',
         alt => [ ],
-        dft => File::Spec->rel2abs( File::Spec->catdir( File::Spec->updir,
-                                                        'perl-inter' ) ),
+        dft => File::Spec->catdir( File::Spec->rel2abs( File::Spec->updir ),
+                                   'perl-inter' ),
         chk => '.+',
     },
     fsync => { 
@@ -231,7 +286,7 @@ my %opt = (
     rsync => {
         msg => 'Which rsync program should be used?',
         alt => [ ],
-        dft => whereis( 'rsync' ),
+        dft => ( whereis( 'rsync' ) || '' ),
     },
     opts => {
         msg => 'Which arguments should be used for rsync?',
@@ -263,7 +318,7 @@ my %opt = (
         msg => "How should the snapshots be extracted?
 Examples:$untarmsg",
         alt => [ ],
-        dft => (get_avail_tar())[0],
+        dft => ( (get_avail_tar())[0] || '' ),
     },
 
     snapext => {
@@ -293,7 +348,7 @@ Examples:$untarmsg",
     unzip => {
         msg => 'How should the patches be unzipped?',
         alt => [ ],
-        dft => whereis( 'gzip' ) . " -cd",
+        dft => ( whereis( 'gzip' ) . " -cd" ),
     },
 
     cleanup => {
@@ -317,10 +372,40 @@ Examples:$untarmsg",
         chk => '.+',
     },
 
+    ftphost => {
+        msg => 'From which server should the source-tree be FTPed',
+        alt => [ ],
+        dft => 'ftp.linux.activestate.com',
+        chk => '.+',
+    },
+
+    ftpusr  => {
+       msg => undef, #no choise
+       alt => [ ],
+       dft => 'anonymous',
+    },
+    ftppwd  => {
+       msg => '',
+       alt => [ ],
+       dft => 'smokers@perl.org',
+    },
+    ftpsdir => {
+       msg => 'Base directory for mirror on FTP server',
+       alt => [ ],
+       dft => '/pub/staff/gsar/APC/perl-current',
+       chk => '.+',
+    },
+    ftpcdir => {
+        msg => 'Base directory for patches on FTP server',
+        alt => [ ],
+        dft => '/pub/staff/gsar/APC/perl-current-diffs',
+        chk => '.+',
+    },
+                 
     patchbin => {
         msg => undef,
         alt => [ ],
-        dft => find_a_patch(),
+        dft => ( find_a_patch() || '' ),
     },
 
     popts => {
@@ -340,7 +425,7 @@ Examples:$untarmsg",
     # make fine-tuning
     makeopt => {
         msg => <<EOT,
-Specify extra arguments for "make".
+Specify extra arguments for "$Config{make}".
 \t(for the build step and test-prep step)
 EOT
         alt => [ ],
@@ -351,14 +436,14 @@ EOT
 Specify a different make program for "make _test".
 EOT
         alt => [ ],
-        dft => 'make',
+        dft => ( $Config{make} ? $Config{make} : 'make' ),
     },
 
     # mail stuff
     mail => {
         msg => "Would you like your reports send by e-mail?",
-        alt => [qw( Y n )],
-        dft => 'y',
+        alt => [qw( N y )],
+        dft => 'n',
     },
     mail_type => {
         msg => 'Which mail facility should be used?',
@@ -375,20 +460,33 @@ EOT
 
     to => {
        msg => <<EOMSG,
-To which address(es) should the report be send?
+To which address(es) should the report *always* be send?
 \t(comma separated list, *please* do not include perl5-porters!)
 EOMSG
        alt => [ ],
        dft => 'smokers-reports@perl.org',
+       nck => '\bperl5-porters@perl.org\b',
     },
 
-    cc => {
+    bcc => {
        msg => <<EOMSG,
-To which address(es) should the report be CCed?
+To which address(es) should the report *always* be BCCed?
 \t(comma separated list, *please* do not include perl5-porters!)
 EOMSG
        alt => [ ],
        dft => '',
+       nck => '\bperl5-porters@perl.org\b',
+    },
+
+    cc => {
+       msg => <<EOMSG,
+* THIS FEATURE HAS CHANGED IN 1.19! *
+To which address(es) should the report be CCed *on fail*?
+\t(comma separated list, *please* do not include perl5-porters!)
+EOMSG
+       alt => [ ],
+       dft => '',
+       nck => '\bperl5-porters@perl.org\b',
     },
 
     ccp5p_onfail => {
@@ -446,11 +544,18 @@ EOT
         alt => [ ],
         dft => "",
     },
+    # Some ENV stuff
+    perl5lib => {
+        msg => "What value should be used for PERL5LIB in the jcl wrapper?
+\t(Make empty, with single space, to not set it.)",
+        alt => [ ],
+        dft => (exists $ENV{PERL5LIB} ? $ENV{PERL5LIB} : ''),
+    },
     # Schedule stuff
     docron => {
         msg => 'Should the smoke be scheduled?',
         alt => [qw( Y n )],
-        dft => 'y',
+        dft => 'n',
     },
     crontime => {
         msg => 'At what time should the smoke be scheduled?',
@@ -508,7 +613,7 @@ There are two additional configuration default files
 F<smoke562_dfconfig> and F<smoke58x_dfconfig> to help you configure 
 B<Test::Smoke> for these two maintenance branches of the source-tree.
 
-To create a configuration for the perl 5.8.x brach:
+To create a configuration for the perl 5.8.x branch:
 
     $ perl configsmoke.pl -p smoke58x
 
@@ -582,30 +687,35 @@ you will need to confirm your choice.
 
 =cut
 
-BUILDDIR: {
-    $arg = 'ddir';
-    $config{ $arg } = prompt_dir( $arg );
-    my $cwd = cwd;
-    unless ( chdir $config{ $arg } ) {
-        warn "Can't chdir($config{ $arg }): $!\n";
-        redo BUILDDIR;
-    }
-    my $bdir = $config{ $arg } = cwd;
-    chdir $cwd or die "Can't chdir($cwd) back: $!\n";
-    if ( $cwd eq $bdir ) {
-        print "The current directory *cannot* be used for smoke testing\n";
-        redo BUILDDIR;
-    }
-
-    $config{ $arg } = File::Spec->canonpath( $config{ $arg } );
-    my $manifest  = File::Spec->catfile( $config{ $arg }, 'MANIFEST' );
-    my $dot_patch = File::Spec->catfile( $config{ $arg }, '.patch' );
-    if ( -e $manifest && -e $dot_patch ) {
-        $opt{use_old}->{dft} = $options{oldcfg} && 
-                               ($conf->{ddir}||"") eq $config{ddir}
-            ? 'y' : $opt{use_old}->{dft};
-        my $use_old = prompt_yn( 'use_old' );
-        redo BUILDDIR unless $use_old;
+{
+    # Hack -des (--usedft) to keep the safeguards
+    local $options{usedft} = $options{usedft};
+    BUILDDIR: {
+        $arg = 'ddir';
+        $config{ $arg } = prompt_dir( $arg );
+        $options{usedft} = $options{oldcfg};
+        my $cwd = cwd;
+        unless ( chdir $config{ $arg } ) {
+            warn "Can't chdir($config{ $arg }): $!\n";
+            redo BUILDDIR;
+        }
+        my $bdir = $config{ $arg } = cwd;
+        chdir $cwd or die "Can't chdir($cwd) back: $!\n";
+        if ( $cwd eq $bdir ) {
+            print "The current directory *cannot* be used for smoke testing\n";
+            redo BUILDDIR;
+        }
+    
+        $config{ $arg } = File::Spec->canonpath( $config{ $arg } );
+        my $manifest  = File::Spec->catfile( $config{ $arg }, 'MANIFEST' );
+        my $dot_patch = File::Spec->catfile( $config{ $arg }, '.patch' );
+        if ( -e $manifest && -e $dot_patch ) {
+            $opt{use_old}->{dft} = $options{oldcfg} && 
+                                   ($conf->{ddir}||"") eq $config{ddir}
+                ? 'y' : $opt{use_old}->{dft};
+            my $use_old = prompt_yn( 'use_old' );
+            redo BUILDDIR unless $use_old;
+        }
     }
 }
 
@@ -616,12 +726,16 @@ There are several build-cfg files provided with the distribution:
 
 =over 4
 
-=item F<perlcurrent.cfg> for 5.8.x+ on unixy systems
+=item F<perlcurrent.cfg> for 5.9.x+ on unixy systems
 
-=item F<w32current.cfg> for 5.8.x+ on MSWin32
+=item F<perl58x.cfg> for 5.8.x (MAINT) on unixy systems
 
 =item F<perl562.cfg> for 5.6.2 (MAINT) on unixy systems
 
+=item F<perl55x.cfg> for 5.005_04 (MAINT) on unixy systems
+
+=item F<w32current.cfg> for 5.8.x+ on MSWin32
+
 =back
 
 Note: 5.6.2 on MSWin32 is not yet provided, but commenting out the
@@ -630,6 +744,7 @@ B<-Duselargefiles> section from F<w32current.cfg> should be enough.
 =cut
 
 $arg = 'cfg';
+default_buildcfg( $config{cfg} || $opt{cfg}{dft}, $config{perl_version} );
 $config{ $arg } = prompt_file( $arg );
 check_buildcfg( $config{ $arg } );
 
@@ -687,9 +802,9 @@ my $want_forest = prompt_yn( $arg );
 FOREST: {
     last FOREST unless $want_forest;
 
-    $config{mdir} = prompt_dir( 'forest_mdir' );
+    $config{mdir} = prompt_dir( 'forest_mdir', 'mdir' );
 
-    $config{fdir} = prompt_dir( 'forest_hdir' );
+    $config{fdir} = prompt_dir( 'forest_hdir', 'fdir' );
 
     $config{sync_type} = 'forest';
 }
@@ -747,7 +862,7 @@ $arg = $want_forest ? 'fsync' : 'sync_type';
 $config{ $arg } = lc prompt( $arg );
 
 SYNCER: {
-    local $_ = $config{ $arg};
+    local *_; $_ = $config{ $arg};
     /^rsync$/ && do {
         $arg = 'source';
         $config{ $arg } = prompt( $arg );
@@ -761,6 +876,15 @@ SYNCER: {
         last SYNCER;
     };
 
+    /^ftp$/  && do {
+        for $arg (qw( ftphost ftpusr ftpsdir ftpcdir )) {
+            $config{ $arg } = prompt( $arg );
+	}
+        $config{ftppwd} = $conf->{ftppwd} || $opt{ftppwd}{dft};
+
+        last SYNCER;
+    };
+
     /^snapshot$/ && do {
         for $arg ( qw( server sdir sfile ) ) {
             if ( $arg ne 'server' && $config{server} =~ m|^https?://|i ) {
@@ -929,17 +1053,18 @@ See L<Test::Smoke::Mailer> and L<mailrpt.pl>
 =cut
 
 $arg = 'mail';
+$opt{ $arg }{dft} = ! $options{usedft};
 $config{ $arg } = prompt_yn( $arg );
 MAIL: {
     last MAIL unless $config{mail};
     $arg = 'mail_type';
     $config{ $arg } = prompt( $arg );
 
-    my $p5p = 'perl5-porters@perl.org';
     $arg = 'to';
-    while ( !$config{ $arg } || $config{ $arg} =~ /\Q$p5p\E/i ) {
-        $config{ $arg } = prompt( $arg ) 
-    }
+    while ( !$config{ $arg } ) { $config{ $arg } = prompt( $arg ) }
+
+    $arg = 'bcc';
+    $config{ $arg } = prompt( $arg );
 
     MAILER: {
         local $_ = $config{ 'mail_type' };
@@ -963,7 +1088,7 @@ MAIL: {
     $config{ $arg } = prompt_yn( $arg );
 
     $arg = 'cc';
-    do {$config{ $arg } = prompt( $arg )} until $config{ $arg } !~ /\Q$p5p\E/i;
+    $config{ $arg } = prompt( $arg );
 }
 
 =item w32args
@@ -1006,6 +1131,7 @@ EO_MSG
         ? "Which make should be used" : undef;
 
     $config{w32make} = prompt( 'w32make' );
+    $config{testmake} = $config{testmake};
 
     $config{w32args} = [ 
         "--win32-cctype" => $config{w32cc},
@@ -1015,6 +1141,28 @@ EO_MSG
     ];
 }
 
+=item vmsmake
+
+Get the make program to use for VMS (MMS or MMK). Start with the one
+this perl was build with.
+
+=cut
+
+VMSMAKE: {
+    is_vms or last VMSMAKE;
+
+    my %vmsmakers = get_avail_vms_make();
+
+    $arg = 'vmsmake';
+    $opt{ $arg } = {
+        msg => "Wich maker should be used?",
+        alt => [ sort keys %vmsmakers ],
+        dft => ( $Config{make} || (sort keys %vmsmakers)[0] ),
+    };
+    $config{ $arg } = prompt( $arg );
+    $config{testmake} = $config{ $arg }
+}
+
 =item make finetuning
 
 Two different config options to accomodate the same thing: 
@@ -1029,7 +1177,7 @@ $arg = 'makeopt';
 $opt{ $arg }->{dft} = '-nologo' if is_win32 && $config{w32make} =~ /nmake/i;
 $config{ $arg } = prompt( $arg );
 
-unless ( is_win32 ) {
+unless ( is_win32 || is_vms ) {
     $arg = 'testmake';
     $config{ $arg } = prompt( $arg ) || 'make';
 }
@@ -1044,7 +1192,7 @@ C<renice> will add a line in the shell-script that starts the smoke.
 
 =cut
 
-unless ( is_win32 ) {
+unless ( is_win32 || is_vms ) {
     $arg = 'umask';
     $config{ $arg } = prompt( $arg );
 
@@ -1089,8 +1237,8 @@ $config{ $arg } = prompt_yn( $arg );
 When C<< $Config{d_alarm} >> is found we can use C<alarm()> to abort 
 long running smokes. Leave this value empty to keep the old behaviour.
 
-    07:30 => F<mktest.pl> is aborted on 7:30 localtime
-   +23:45 => F<mktest.pl> is aborted after 23 hours and 45 minutes
+    07:30 => F<smokeperl.pl> is aborted on 7:30 localtime
+   +23:45 => F<smokeperl.pl> is aborted after 23 hours and 45 minutes
 
 Thank you Jarkko for donating this suggestion.
 
@@ -1113,7 +1261,37 @@ If you want this then set the directory where you want the stored
 
 $arg = 'adir';
 $config{ $arg } = prompt_dir( $arg );
-$config{lfile} = $options{log} unless $config{ $arg } eq "";
+$config{lfile} = File::Spec->rel2abs( $options{log}, cwd );
+
+=item delay_report
+
+Some filesystems do not support opening an already opened file. This
+makes it hard to scan the logfile for compiler messages. We can delay
+the creation of the report and call F<mailrpt.pl> after
+F<smokeperl.pl>. MSWin32 and VMS might benefit.
+
+=cut
+
+$arg = 'delay_report';
+$config{ $arg } = $^O =~ /VMS/;
+
+=item ENV stuff
+
+If you have a value for PERL5LIB set in the config environment, you
+could have it transferred tho the jcl-wrapperscript. Do not bother
+asking if it is not there.
+
+=cut
+
+my $has_perl5lib = exists $ENV{PERL5LIB} && defined $ENV{PERL5LIB} &&
+                   length $ENV{PERL5LIB};
+
+P5LIB: {
+    $has_perl5lib or last P5LIB;
+    print "\nI see you have PERL5LIB set to: '$ENV{PERL5LIB}'";
+    $arg = 'perl5lib';
+    $config{ $arg } = prompt( $arg );
+}
 
 =item schedule stuff
 
@@ -1135,7 +1313,8 @@ as F<at.exe> has not got a way comment-out entries.
 =cut
 
 my( $cron, $has_crond,  $crontime );
-SCHEDULE: { 
+SCHEDULE: {
+    is_vms and last SCHEDULE;
     ( $cron, $has_crond ) = get_avail_scheduler();
 
     last SCHEDULE unless $cron;
@@ -1210,23 +1389,31 @@ SAVEALL: {
     save_config();
     if ( is_win32 ) {
         $jcl = write_bat();
+    } elsif ( is_vms ) {
+        $jcl = write_com();
     } else {
         $jcl = write_sh();
     }
 }
 
-print <<EOMSG;
+WRAPUP: {
+    local $" = "";
+    my $chkbcfg = File::Spec->catfile( $findbin, 'chkbcfg.pl' );
+    print <<EOMSG;
+Finished configuration:
 
-Run the perl core test smoke suite with:
+* Please check "$config{cfg}" for the
+  configurations you want to test:
+@{ [map "    $_" => qx( $^X $chkbcfg $config{cfg} )] }
+
+* Run the perl core test smoke suite with:
 \t$jcl
 
-Please check "$config{cfg}" 
-for the configurations you want to test.
-
-Have the appropriate amount of fun!
+* Have the appropriate amount of fun!
 
                                     The Test::Smoke team.
 EOMSG
+}
 
 =back
 
@@ -1272,7 +1459,8 @@ sub sort_configkeys {
 
         # Sync related
         qw( sync_type fsync rsync opts source tar server sdir sfile
-            patchup pserver pdir unzip patchbin cleanup cdir hdir pfile ),
+            patchup pserver pdir unzip patchbin cleanup cdir hdir pfile
+            ftphost ftpusr ftppwd ftpsdir ftpcdir ),
 
         # OS specific make related
         qw( w32args w32cc w32make ),
@@ -1281,13 +1469,16 @@ sub sort_configkeys {
         qw( force_c_locale locale defaultenv ),
 
         # Report related
-        qw( mail mail_type mserver from to ccp5p_onfail cc ),
+        qw( mail mail_type mserver from to ccp5p_onfail cc bcc ),
 
-        # Archive report and logfile
+        # Archive reports and logfile
         qw( adir lfile ),
 
         # make fine-tuning
         qw( makeopt testmake ),
+
+        # ENV stuff
+        qw( perl5lib delay_report ),
     );
 
     my $i = 0;
@@ -1310,9 +1501,15 @@ C<write_sh()> creates the shell-script.
 sub write_sh {
     my $cwd = cwd();
     my $jcl = "$options{jcl}.sh";
-    my $smokeperl = File::Spec->catfile( $FindBin::Bin, 'smokeperl.pl' );
+    my $smokeperl = File::Spec->catfile( $findbin, 'smokeperl.pl' );
     my $cronline = schedule_entry( File::Spec->catfile( $cwd, $jcl ), 
                                    $cron, $crontime );
+
+    my $p5lib = $config{perl5lib} ? <<EO_P5L : '';
+PERL5LIB=$config{perl5lib}
+export PERL5LIB
+EO_P5L
+
     my $handle_lock = $config{killtime} ? <<EO_CONT : <<EO_DIE;
     # Not sure about this, so I will keep the old behaviour 
     # smokeperl.pl will exit(42) on timeout
@@ -1346,7 +1543,8 @@ $handle_lock
 fi
 echo "\$CFGNAME" > "\$LOCKFILE"
 
-PATH=$FindBin::Bin:$ENV{PATH}
+$p5lib
+PATH=$findbin:$ENV{PATH}
 export PATH
 umask $config{umask}
 $^X $smokeperl -c "\$CFGNAME" \$continue \$\* > $options{log} 2>&1
@@ -1370,18 +1568,33 @@ because it uses commands that are not supported by B<COMMAND.COM>
 
 sub write_bat {
     my $cwd = File::Spec->canonpath( cwd() );
+    my $findbin_bin = File::Spec->canonpath( $findbin );
 
-    my $smokeperl = File::Spec->catfile( $FindBin::Bin, 'smokeperl.pl' );
-    my $copycmd = $config{w32args}->[1] ne "BORLAND" ? "" : <<EOCOPYCMD;
+    my $smokeperl  = File::Spec->catfile( $findbin_bin, 'smokeperl.pl' );
+    my $archiverpt = File::Spec->catfile( $findbin_bin, 'archiverpt.pl' );
+    my $mailrpt    = File::Spec->catfile( $findbin_bin, 'mailrpt.pl' );
+    my $copycmd = <<'EOCOPYCMD';
 
-REM I found hanging XCOPY while smoking with BORLAND
-set COPYCMD=/Y %COPYCMD%
+REM I found hanging XCOPY while smoking; uncommenting the next line might help
+REM set COPYCMD=/Y \%COPYCMD\%
 
 EOCOPYCMD
+    my $p5lib = $config{perl5lib} ? <<EO_P5L : '';
+
+set PERL5LIB=$config{perl5lib}
+EO_P5L
+
 
     my $jcl = "$options{jcl}.cmd";
     my $atline = schedule_entry( File::Spec->catfile( $cwd, $jcl ), 
                                  $cron, $crontime );
+
+    my $archive = qq/$^X $archiverpt -c "\%CFGNAME\%"/;
+    $config{lfile} or $archive =~ s/^/REM /;
+
+    my $report = qq/$^X $mailrpt -c "\%CFGNAME\%"/;
+    $config{delay_report} or $report =~ s/^/REM /;
+
     local *MYSMOKEBAT;
     open MYSMOKEBAT, "> $jcl" or
         die "Cannot write '$jcl': $!";
@@ -1394,6 +1607,7 @@ REM @{[ scalar localtime ]}
 REM NOTE: Changes made in this file will be \*lost\*
 REM       after rerunning $0
 $copycmd
+$p5lib
 REM $atline
 
 set WD=$cwd\\
@@ -1411,9 +1625,10 @@ if NOT EXIST \%LOCKFILE\% goto START_SMOKE
 :START_SMOKE
     echo \%CFGNAME\% > \%LOCKFILE\%
     set OLD_PATH=\%PATH\%
-    set PATH=$FindBin::Bin;\%PATH\%
+    set PATH=$findbin_bin;\%PATH\%
     $^X $smokeperl -c "\%CFGNAME\%" \%* > "\%WD\%\\$options{log}" 2>&1
-
+    $archive
+    $report
     set PATH=\%OLD_PATH\%
 
 del \%LOCKFILE\%
@@ -1425,18 +1640,57 @@ EO_BAT
     return File::Spec->canonpath( File::Spec->rel2abs( $jcl ) );
 }
 
+=item write_com
+
+Write a simple DCL script that helps running the smoke suite.
+
+=cut
+
+sub write_com {
+    my $jcl = "$options{jcl}.com";
+    my $cwd = File::Spec->canonpath( cwd() );
+    my $p5lib_p = $config{perl5lib} ? ' '  : '!';
+    my $p5lib = "$p5lib_p DEFINE PERL5LIB $config{perl5lib}";
+    local *MYSMOKECOM;
+    open MYSMOKECOM, "> $jcl" or
+        die "Cannot write '$jcl': $!";
+    print MYSMOKECOM <<EO_COM;
+\$! $jcl - Run Perl5 core test smoke suite
+\$!
+\$! Written by $0 v$VERSION
+\$! @{[ scalar localtime ]}
+\$! NOTE: Changes made in this file will be \*lost\*
+\$!       after rerunning $0
+\$! Try:
+\$!       SUBMIT/NOPRINTER/NOTIFY $cwd$jcl
+\$!
+\$  SET DEFAULT $cwd
+\$$p5lib
+\$! DEFINE/USER sys\$output $options{log}
+\$! DEFINE/USER sys\$error $options{log}
+\$  MCR $^X ${findbin}smokeperl.pl "-c=$options{config}"
+EO_COM
+    close MYSMOKECOM or warn "Error writing '$jcl': $!";
+
+    print "Finished writing '$jcl'\n";
+
+    return '@' . File::Spec->canonpath( File::Spec->rel2abs( $jcl ) );
+}
+
 sub prompt {
-    my( $message, $alt, $df_val, $chk ) = 
-        @{ $opt{ $_[0] } }{qw( msg alt dft chk )};
+    my( $message, $alt, $df_val, $chk, $nck ) = 
+        @{ $opt{ $_[0] } }{qw( msg alt dft chk nck )};
 
     $df_val = $conf->{ $_[0] } if exists $conf->{ $_[0] };
+    $df_val = $conf->{ $_[1] } if $_[1] && exists $conf->{ $_[1] };
     unless ( defined $message ) {
         my $retval = defined $df_val ? $df_val : "undef";
         (caller 1)[3] or print "Got [$retval]\n";
         return $df_val;
     }
-    $message =~ s/\s+$//;
 
+    $message =~ s/\s+$//;
+        
     my %ok_val;
     %ok_val = map { (lc $_ => 1) } @$alt if @$alt;
     $chk ||= '.*';
@@ -1450,8 +1704,12 @@ sub prompt {
 
     my( $input, $clear );
     INPUT: {
-        print "$alts\[$default] \$ ";
-        chomp( $input = <STDIN> );
+        if ( $options{usedft} ) {
+            $input = defined $df_val ? $df_val : " ";
+        } else {
+            print "$alts\[$default] \$ ";
+            chomp( $input = <STDIN> );
+        }
         if ( $input eq " " ) {
             $input = "";
             $clear = 1;
@@ -1461,7 +1719,10 @@ sub prompt {
             $input = $df_val unless length $input;
         }
 
-        printf "Input does not match $chk\n" and redo INPUT
+        print "Input should not match: '/$nck/i'\n" and redo INPUT
+            if $nck && $input =~ m/$nck/i;
+        
+        print "Input does not match: 'm/$chk/'\n" and redo INPUT
             unless $input =~ m/$chk/i;
 
         last INPUT unless %ok_val;
@@ -1550,6 +1811,9 @@ sub prompt_yn {
 sub whereis {
     my( $prog, $find_all ) = @_;
     return '' unless $prog; # you shouldn't call it '0'!
+    is_vms and return vms_whereis( $prog, $find_all );
+
+    $ENV{PATH} or return wantarray ? ( ) : [ ];
 
     my $p_sep = $Config::Config{path_sep};
     my @path = split /\Q$p_sep\E/, $ENV{PATH};
@@ -1569,6 +1833,50 @@ sub whereis {
     return @fnames ? wantarray ? @fnames : \@fnames : '';
 }
 
+=item vms_whereis( $prog )
+
+First look in the SYMBOLS to see if C<$prog> is there.
+Next look in the KFE-table C<INSTALL LIST> if it is there.
+As a last resort we can scan C<DCL$PATH> like we do on *nix/Win32
+
+=cut
+
+sub vms_whereis {
+    my( $prog, $find_all ) = @_;
+
+    # Check SYMBOLS
+    eval { require VMS::DCLsym };
+    if ( $@ ) {
+        carp "Oops, cannot load VMS::DCLsym: $@";
+    } else {
+        VMS::DCLsym->import;
+        my $syms = VMS::DCLsym->new;
+        return $prog if scalar $syms->getsym( uc $prog );
+    }
+    # Check Known File Entry table (INSTALL LIST)
+    my $img_re = '^\s+([\w\$]+);\d+';
+    my %kfe = map {
+        my $img = /$img_re/ ? $1 : '';
+        ( uc $img => undef )
+    } grep /$img_re/ => qx/INSTALL LIST/;
+    return $prog if exists $kfe{ uc $prog };
+
+    my $dclp_env = 'DCL$PATH';
+    my @path = $ENV{ $dclp_env }
+        ?split /$Config{path_sep}/, $ENV{ $dclp_env } : ();
+    my @pext = ( $Config{exe_ext} || $Config{_exe}, '.COM' );
+
+    foreach my $dir ( @path ) {
+        foreach my $ext ( @pext ) {
+            my $fname = File::Spec->catfile( $dir, "$prog$ext" );
+            if ( -x $fname ) {
+                return $ext eq '.COM' ? "\@$fname" : "$fname";
+            }
+        }
+    }
+    return '';
+}
+
 sub find_a_patch {
 
     my $patch_bin;
@@ -1606,6 +1914,9 @@ sub get_avail_sync {
     # (has_ftp && 5.9.x) || has_lwp
     unshift @synctype, 'snapshot' 
         if ( $has_ftp && $pversion eq '5.9.x' ) || $has_lwp;
+
+    unshift @synctype, 'ftp' if $has_ftp;
+
     unshift @synctype, 'rsync' if whereis( 'rsync' );
     return @synctype;
 }
@@ -1641,7 +1952,14 @@ sub tar_fmt {
 }
 
 sub check_locale {
-    # I only know one way...
+    # I only know one way... and one for Darwin (perhaps FreeBSD)
+    if ( $^O =~ /darwin|bsd/i ) {
+        local *USL;
+        opendir USL, '/usr/share/locale' or return;
+        my @list = grep /utf-?8$/i => readdir USL;
+        closedir USL;
+        return @list;
+    } 
     my $locale = whereis( 'locale' );
     return unless $locale;
     return grep /utf-?8$/i => split /\n/, `$locale -a`;
@@ -1740,15 +2058,55 @@ sub get_avail_w32compilers {
     } grep length $map{ $_ }->{ccbin} => keys %map;
 }
 
+sub get_avail_vms_make {
+
+    return map +( $_ => undef ) => grep defined $_ && length( $_ ) 
+        => map vms_whereis( $_ ) => qw( MMK MMS );
+ 
+    local *QXERR; open *QXERR, ">&STDERR"; close STDERR;
+
+    my %makers = map {
+        my $maker = $_;
+        map +( $maker => /V([\d.-]+)/ ? $1 : '' )
+        => grep /\b$maker\b/ && /V[\d.-]+/ => qx($maker/IDENT)
+    } qw( MMK MMS );
+
+    open STDERR, ">&QXERR"; close QXERR;
+
+    return %makers;
+}
+
 sub get_Win_version {
-    my @osversion = Win32::GetOSVersion();
-
-    my $win_version = join '.', @osversion[ 1, 2 ];
-    $win_version .= " $osversion[0]" if $osversion[0];
-
+    require Test::Smoke::SysInfo;
+    ( my $win_version = Test::Smoke::SysInfo::__get_os() ) =~ s/^[^-]*- //;
     return $win_version;
 }
 
+=item default_buildcfg( $file_name, $pversion )
+
+Check to see if C<$file_name> exists. If not, copy the default config
+for C<$pversion> to C<$file_name>.
+
+=cut
+
+sub default_buildcfg {
+    my( $file_name, $pversion ) = @_;
+    -f $file_name and return 1;
+
+    $pversion =~ tr/.//d;
+    $pversion eq '59x' and $pversion = 'current';
+    my $basename = is_win32
+        ? "w32current.cfg"
+        : is_vms ? "vmsperl.cfg" : "perl${pversion}.cfg";
+
+    my $dftbcfg = File::Spec->catfile( $findbin, $basename );
+    -f $dftbcfg 
+        or die "You seem to have an incomplete Test::Smoke installation" . 
+               "($dftbcfg is missing)!\n";
+    copy $dftbcfg, $file_name
+        and print "\nCreated buildconfig '$file_name'";
+}
+
 =item check_buildcfg
 
 We will try to check the build configurations file to see if we should
@@ -1769,7 +2127,10 @@ sub check_buildcfg {
     close BCFG;
     my $oldcfg = join "", grep !/^#/ => @bcfg;
 
-    my @no_option = ( );
+    my ($rev, @vparts ) = $config{perl_version} =~ /^(\d)(?:\.(\d+))+/;
+    my $pversion = sprintf "%d.%03d%02d", $rev, @vparts, 0;
+    print "Checking '$file_name' ($pversion/$^O)\n";
+    my @no_option = $pversion >= 5.009 ? ( '-Uuseperlio' ) : ( );
     OSCHECK: {
         local $_ = $^O;
         /darwin|bsd/i && do { 
@@ -1822,6 +2183,7 @@ write the new one in its place.
 sub finish_cfgcheck {
     my( $overwrite, $fname, $bcfg ) = @_;
 
+    my $msg = "";
     if ( $overwrite ) {
         my $backup = "$fname.bak";
         -f $backup and chmod( 0775, $backup ) and unlink $backup;
@@ -1829,6 +2191,7 @@ sub finish_cfgcheck {
             warn "Cannot rename '$fname' to '$backup': $!";
     } else {
         $fname = "$options{prefix}.cfg";
+        $msg = " (in case you want it later)";
     }
     # change the filemode (make install used to make perlcurrent.cfg readonly)
     -f $fname and chmod 0775, $fname;
@@ -1841,7 +2204,7 @@ sub finish_cfgcheck {
         warn "Error on close '$fname': $!";
         return;
     };
-    print "Wrote '$fname'\n";
+    print "Wrote '$fname'$msg\n";
 }
 
 =back
@@ -1854,7 +2217,7 @@ Schedule, logfile optional
 
 In case I forget to update the C<$VERSION>:
 
-    $Id: configsmoke.pl 482 2003-10-20 05:36:13Z abeltje $
+    $Id: configsmoke.pl 872 2005-07-21 16:53:21Z abeltje $
 
 =head1 COPYRIGHT
 
