@@ -1,31 +1,34 @@
 package Test::Smoke::Mailer;
 use strict;
 
-# $Id: Mailer.pm 458 2003-09-28 10:57:04Z abeltje $
-use vars qw( $VERSION $P5P);
-$VERSION = '0.007';
+# $Id: Mailer.pm 785 2004-11-11 06:12:23Z abeltje $
+use vars qw( $VERSION $P5P $NOCC_RE);
+$VERSION = '0.010';
 
 use Test::Smoke::Util qw( parse_report_Config );
 
-$P5P = 'perl5-porters@perl.org';
+$P5P       = 'perl5-porters@perl.org';
+$NOCC_RE   = ' (?:PASS\b|FAIL\(X\))';
 my %CONFIG = (
     df_mailer        => 'Mail::Sendmail',
     df_ddir          => undef,
     df_v             => 0,
+    df_rptfile       => 'mktest.rpt',
     df_to            => 'daily-build-reports@perl.org',
     df_from          => '',
     df_cc            => '',
+    df_bcc           => '',
     df_ccp5p_onfail  => 0,
     df_mserver       => 'localhost',
 
     df_mailbin       => 'mail',
-    mail             => [qw( cc mailbin )],
+    mail             => [qw( bcc cc mailbin )],
     df_mailxbin      => 'mailx',
-    mailx            => [qw( cc mailxbin )],
+    mailx            => [qw( bcc cc mailxbin )],
     df_sendmailbin   => 'sendmail',
-    sendmail         => [qw( from cc sendmailbin )],
-    'Mail::Sendmail' => [qw( from cc mserver )],
-    'MIME::Lite'     => [qw( from cc mserver )],
+    sendmail         => [qw( from bcc cc sendmailbin )],
+    'Mail::Sendmail' => [qw( from bcc cc mserver )],
+    'MIME::Lite'     => [qw( from bcc cc mserver )],
 
     valid_mailer => { sendmail => 1, mail => 1, mailx => 1,
                       'Mail::Sendmail' => 1, 'MIME::Lite' => 1, },
@@ -88,7 +91,7 @@ sub  new {
     my %fields = map {
         my $value = exists $args{$_} ? $args{ $_ } : $CONFIG{ "df_$_" };
         ( $_ => $value )
-    } ( v => ddir => to => ccp5p_onfail => @{ $CONFIG{ $mailer } } );
+    } ( rptfile => v => ddir => to => ccp5p_onfail => @{ $CONFIG{ $mailer } } );
     $fields{ddir} = File::Spec->rel2abs( $fields{ddir} );
 
     DO_NEW: {
@@ -114,7 +117,7 @@ subject line for the mail-message.
 sub fetch_report {
     my $self = shift;
 
-    my $report_file = File::Spec->catfile( $self->{ddir}, 'mktest.rpt' );
+    my $report_file = File::Spec->catfile( $self->{ddir}, $self->{rptfile} );
 
     local *REPORT;
     if ( open REPORT, "< $report_file" ) {
@@ -146,17 +149,19 @@ sub error {
 
 C<_get_cc()> implements the C<--ccp5p_onfail> option. It looks at the
 subject to see if the smoke FAILed and then adds the I<perl5-porters>
-mailing-list to the C<Cc:> field, unless it is already in C<To:> or
-C<Cc:>.
+mailing-list to the C<Cc:> field unless it is already part of C<To:>
+or C<Cc:>.
+
+The new behaviour is to only return C<Cc:> on fail. This is determined
+by the new global regex kept in C<< $Test::Smoke::Mailer::NOCC_RE >>.
 
 =cut
 
 sub _get_cc {
     my( $self, $subject ) = @_;
-    return $self->{cc} || "" unless $self->{ccp5p_onfail};
+    return "" if $subject =~ m/$NOCC_RE/;
 
-    # State all cases for which p5p should be added
-    return $self->{cc} || "" unless $subject !~ / PASS\b/;
+    return $self->{cc} || "" unless $self->{ccp5p_onfail};
 
     my $p5p = $Test::Smoke::Mailer::P5P or return $self->{cc};
     my @cc = $self->{cc} ? $self->{cc} : ();
@@ -248,6 +253,7 @@ sub mail {
     $header   .= "From: $self->{from}\n" 
         if exists $self->{from} && $self->{from};
     $header   .= "Cc: $cc\n" if $cc;
+    $header   .= "Bcc: $self->{bcc}\n" if $self->{bcc};
     $header   .= "Subject: $subject\n\n";
 
     $self->{v} > 1 and print "[$self->{sendmailbin} -i -t]\n";
@@ -315,6 +321,7 @@ sub mail {
 
     my $cmdline = qq|$mailer -s '$subject'|;
     $cmdline   .= qq| -c '$cc'| if $cc;
+    $cmdline   .= qq| -b '$self->{bcc}'| if $self->{bcc};
     $cmdline   .= qq| $self->{to}|;
 
     $self->{v} > 1 and print "[$cmdline]\n";
@@ -388,6 +395,7 @@ sub mail {
         Body    => $self->{body},
     );
     $message{cc}   = $cc if $cc;
+    $message{bcc}   = $self->{bcc} if $self->{bcc};
     $message{from} = $self->{from} if $self->{from};
     $message{smtp} = $self->{mserver} if $self->{mserver};
 
@@ -459,6 +467,7 @@ sub mail {
         Data    => $self->{body},
     );
     $message{Cc}   = $cc  if $cc;
+    $message{Bcc}   = $self->{bcc} if $self->{bcc};
     $message{From} = $self->{from} if $self->{from};
     MIME::Lite->send( smtp => $self->{mserver} ) if $self->{mserver};
 
