@@ -2,7 +2,7 @@
 use strict;
 $|=1;
 
-# $Id: smokeperl.pl 480 2003-10-19 13:21:09Z abeltje $
+# $Id: smokeperl.pl 867 2005-07-21 14:04:44Z abeltje $
 use vars qw( $VERSION );
 $VERSION = Test::Smoke->VERSION;
 
@@ -17,15 +17,17 @@ use Config;
 use Test::Smoke::Syncer;
 use Test::Smoke::Patcher;
 use Test::Smoke;
+use Test::Smoke::Reporter;
 use Test::Smoke::Mailer;
 use Test::Smoke::Util qw( get_patch calc_timeout do_pod2usage );
 
 use Getopt::Long;
 Getopt::Long::Configure( 'pass_through' );
-my %options = ( config => 'smokecurrent_config', run => 1,
+my %options = ( config => 'smokecurrent_config', run => 1, pfile => undef,
                 fetch => 1, patch => 1, mail => undef, archive => undef,
-                continue => 0, ccp5p_onfail => undef,
-                is56x => undef, defaultenv => undef, smartsmoke => undef );
+                continue => 0, ccp5p_onfail => undef, killtime => undef,
+                is56x => undef, defaultenv => undef, smartsmoke => undef,
+                delay_report => undef, v => undef );
 
 my $myusage = "Usage: $0 [-c configname]";
 GetOptions( \%options, 
@@ -34,6 +36,7 @@ GetOptions( \%options,
     'patch!',
     'ccp5p_onfail!',
     'mail!',
+    'delay_report!',
     'run!',
     'archive!',
     'is56x',
@@ -41,7 +44,10 @@ GetOptions( \%options,
     'continue',
     'smartsmoke!',
     'snapshot|s=i',
+    'killtime=s',
+    'pfile=s',
 
+    'v|verbose=i',
     'help|h', 'man',
 ) or do_pod2usage(  verbose => 1, myusage => $myusage );
 
@@ -72,12 +78,16 @@ It can take these options
   --nomail                 Skip the mail step
   --noarchive              Skip the archive step (if applicable)
   --[no]ccp5p_onfail       Do (not) send failure reports to perl5-porters
+  --[no]delay_report       Do (not) create the report now
 
   --continue               Try to continue an interrupted smoke
   --is56x                  This is a perl-5.6.x smoke
   --defaultenv             Run a smoke in the default environment
   --[no]smartsmoke         Don't smoke unless patchlevel changed
   --snapshot <patchlevel>  Set a new patchlevel for snapshot smokes
+  --killtime (+)hh::mm     (Re)set the guard-time for this smoke
+
+  --pfile <patchesfile>    Set a patches-to-apply-file
 
 All other arguments are passed to F<Configure>!
 
@@ -97,13 +107,23 @@ unless ( read_config( $config_file ) ) {
 defined Test::Smoke->config_error and 
     die "!!!Please run 'configsmoke.pl'!!!\nCannot find configuration: $!";
 
+# smartsmoke doesn't make sense with nofetch (unless you say so)
+defined $options{fetch} && !$options{fetch} && !defined $options{smartsmoke}
+    and $options{smartsmoke} = 0;
+
 # Correction for backward compatability
 !defined $options{ $_ } && !exists $conf->{ $_ } and $options{ $_ } = 1
-    for qw( run fetch patch mail archive );
+    for qw( run fetch patch mail archive v );
+!defined $options{ $_ } && !exists $conf->{ $_ } and $options{ $_ } = 0
+    for qw( delay_report );
+
 # Make command-line options override configfile
 defined $options{ $_ } and $conf->{ $_ } = $options{ $_ }
-    for qw( is56x defaultenv continue
-            smartsmoke run fetch patch mail ccp5p_onfail archive );
+    for qw( is56x defaultenv continue killtime pfile delay_report
+            smartsmoke run fetch patch mail ccp5p_onfail archive v );
+
+# Make sure the --pfile command-line override works
+$options{pfile} and $conf->{patch_type} ||= 'multi';
 
 if ( $options{continue} ) {
     $options{v} and print "Will try to continue current smoke\n";
@@ -115,8 +135,12 @@ if ( $options{continue} ) {
 my $cwd = cwd();
 chdir $conf->{ddir} or die "Cannot chdir($conf->{ddir}): $!";
 call_mktest();
-call_mkovz();
-mailrpt();
+unless ( $conf->{delay_report} ) {
+    genrpt();
+    mailrpt();
+} else {
+    $conf->{v} and print "Delayed creation of the report. See 'mailrpt.pl'\n";
+}
 chdir $cwd;
 archiverpt();
 
@@ -181,7 +205,7 @@ sub call_mktest {
     }
     $timeout and local $SIG{ALRM} = sub {
         warn "This smoke is aborted ($conf->{killtime})\n";
-        call_mkovz();
+        genrpt();
         mailrpt();
         exit(42);
     };
@@ -190,14 +214,10 @@ sub call_mktest {
     run_smoke( $conf->{continue}, @ARGV );
 }
 
-sub call_mkovz {
+sub genrpt {
     return unless $options{run};
-    local @ARGV = ( 'nomail', $conf->{ddir} );
-    push  @ARGV, $conf->{locale} ? $conf->{locale} : "";
-    push  @ARGV, $conf->{defaultenv} if $conf->{defaultenv};
-    my $mkovz = File::Spec->catfile( $FindBin::Bin, 'mkovz.pl' );
-    local $0 = $mkovz;
-    do $mkovz or die "Error in mkovz.pl: $@";
+    my $reporter = Test::Smoke::Reporter->new( $conf );
+    $reporter->write_to_file;
 }
 
 sub mailrpt {
@@ -234,8 +254,8 @@ sub archiverpt {
         my $archived_log = "log${patch_level}.log";
         last SKIP_LOG unless defined $conf->{lfile};
         last SKIP_LOG 
-            unless -f File::Spec->catfile( $FindBin::Bin, $conf->{lfile} );
-        copy( File::Spec->catfile( $FindBin::Bin, $conf->{lfile} ),
+            unless -f $conf->{lfile};
+        copy( $conf->{lfile},
               File::Spec->catfile( $conf->{adir}, $archived_log ) ) or
             die "Cannot copy to '$archived_log': $!";
     }
@@ -260,7 +280,7 @@ L<README>, L<FAQ>, L<configsmoke.pl>, L<mktest.pl>, L<mkovz.pl>
 
 =head1 REVISION
 
-$Id: smokeperl.pl 480 2003-10-19 13:21:09Z abeltje $
+$Id: smokeperl.pl 867 2005-07-21 14:04:44Z abeltje $
 
 =head1 COPYRIGHT
 
